# -*- coding: utf-8 -*-

"""Regression tests."""
import os
from unittest import TestCase

from morelia.decorators import tags
from morelia.parser import Parser

pwd = os.path.dirname(os.path.realpath(__file__))


@tags(['acceptance', 'regression'])
class SetUpTearDownTest(TestCase):

    """Test for recreating bug with multiple setUp/tearDown call."""

    def setUp(self):
        try:
            self._setup_count += 1
        except AttributeError:
            self._setup_count = 1

    def tearDown(self):
        try:
            self._teardown_count += 1
        except AttributeError:
            self._teardown_count = 1

    def test_setup_teardown(self):
        """Check for multiple setUp/tearDown calls."""
        filename = os.path.join(pwd, 'features/setupteardown.feature')
        ast = Parser().parse_file(filename)
        ast.evaluate(self, show_all_missing=True)

    def test_many_test_methods(self):
        """Check setUp/tearDown when many tests in one TestCase."""
        filename = os.path.join(pwd, 'features/setupteardown.feature')
        ast = Parser().parse_file(filename)
        ast.evaluate(self, show_all_missing=True)

    def test_not_morelia_test(self):
        """Check if influcence on non morelia tests."""
        setup_count = getattr(self, '_setup_count', 0)
        teardown_count = getattr(self, '_teardown_count', 0)
        self.assertEqual(setup_count, 1)
        self.assertEqual(teardown_count, 0)

    def step_step_one(self):
        r'step one'

        setup_count = getattr(self, '_setup_count', 0)
        teardown_count = getattr(self, '_teardown_count', 0)
        self.assertEqual(setup_count, 1)
        self.assertEqual(teardown_count, 0)

    def step_step_two(self):
        r'step two'

        setup_count = getattr(self, '_setup_count', 0)
        teardown_count = getattr(self, '_teardown_count', 0)
        self.assertEqual(setup_count, 2)
        self.assertEqual(teardown_count, 1)


@tags(['acceptance', 'regression'])
class EncodingErrorInTraceback(TestCase):

    def test_should_report_on_all_failing_scenarios(self):
        filename = os.path.join(pwd, 'features/encoding_error_in_regression.feature')
        ast = Parser().parse_file(filename)
        self.assertRaisesRegexp(AssertionError, 'Given Zażółć gęślą jaźń', ast.evaluate, self)

    def step_failing(self):
        r'Zażółć gęślą jaźń'

        assert False, u'Zażółć gęślą jaźń Эх, чужак! Общий съём цен шляп (юфть) — вдрызг!'



import inspect
import sys
import traceback
from gettext import ngettext
from abc import ABCMeta, abstractmethod
from collections import OrderedDict
import time

from .grammar import Feature, Scenario, Step
from .exceptions import MissingStepError
from .utils import to_docstring, fix_exception_encoding, to_unicode


class IVisitor(object):

    __metaclass__ = ABCMeta

    def permute_schedule(self, node):
        return [[0]]

    @abstractmethod
    def visit(self, node):
        pass  # pragma: nocover

    @abstractmethod
    def after_visit(self, node):
        pass  # pragma: nocover


def noop():
    pass


class TestVisitor(IVisitor):
    """ Visits all steps and run step methods. """

    def __init__(self, suite, matcher, formatter):
        self._setUp = suite.setUp
        self._tearDown = suite.tearDown
        self._suite = suite
        self._suite.setUp = self._suite.tearDown = noop
        self._matcher = matcher
        self._formatter = formatter
        self._exceptions = []
        self._scenarios_failed = 0
        self._scenarios_passed = 0
        self._scenarios_num = 0
        self._scenario_exception = None
        self._steps_num = 0

    def visit(self, node):
        # no support for single dispatch for methods in python yet
        if isinstance(node, Feature):
            self._feature_visit(node)
        elif isinstance(node, Scenario):
            self._scenario_visit(node)
        elif isinstance(node, Step):
            self._step_visit(node)
        else:
            line = node.get_real_reconstruction()
            self._formatter.output(node, line, '', 0)

    def after_visit(self, node):
        if isinstance(node, Feature):
            self._feature_after_visit(node)
        elif isinstance(node, Scenario):
            self._scenario_after_visit(node)

    def _feature_visit(self, node):
        self._exceptions = []
        self._scenarios_failed = 0
        self._scenarios_passed = 0
        self._scenarios_num = 0
        line = node.get_real_reconstruction()
        self._formatter.output(node, line, '', 0)

    def _feature_after_visit(self, node):
        if self._scenarios_failed:
            self._fail_feature()

    def _fail_feature(self):
        failed_msg = ngettext('{} scenario failed', '{} scenarios failed', self._scenarios_failed)
        passed_msg = ngettext('{} scenario passed', '{} scenarios passed', self._scenarios_passed)
        msg = u'{}, {}'.format(failed_msg, passed_msg).format(self._scenarios_failed, self._scenarios_passed)
        prefix = '-' * 66
        for step_line, tb, exc in self._exceptions:
            msg += u'\n{}{}\n{}{}'.format(prefix, step_line, tb, exc).replace(u'\n', u'\n    ')
        assert self._scenarios_failed == 0, msg

    def _scenario_visit(self, node):
        self._scenario_exception = None
        if self._scenarios_num != 0:
            self._setUp()
        self._scenarios_num += 1
        line = node.get_real_reconstruction()
        self._formatter.output(node, line, '', 0)

    def _scenario_after_visit(self, node):
        if self._scenario_exception:
            self._exceptions.append(self._scenario_exception)
            self._scenarios_failed += 1
        else:
            self._scenarios_passed += 1
        self._tearDown()

    def _step_visit(self, node):
        if self._scenario_exception:
            return
        self._suite.step = node
        self._steps_num += 1
        reconstruction = node.get_real_reconstruction()
        start_time = time.time()
        status = 'pass'
        try:
            self.run_step(node)
        except (MissingStepError, AssertionError) as exc:
            status = 'fail'
            etype, evalue, etraceback = sys.exc_info()
            tb = traceback.extract_tb(etraceback)[:-2]
            fix_exception_encoding(evalue)
            self._scenario_exception = (
                node.parent.get_real_reconstruction() + reconstruction,
                ''.join(to_unicode(line) for line in traceback.format_list(tb)),
                ''.join(to_unicode(line) for line in traceback.format_exception_only(etype, evalue))
            )
        except (SystemExit, Exception) as exc:
            status = 'error'
            self._format_exception(node, exc)
            raise
        finally:
            end_time = time.time()
            duration = end_time - start_time
            self._formatter.output(node, reconstruction, status, duration)

    def _format_exception(self, node, exc):
        if len(exc.args):
            message = node.format_fault(exc.args[0])
            exc.args = (message,) + exc.args[1:]

    def run_step(self, node):
        method, args, kwargs = node.find_step(self._matcher)
        spec = inspect.getargspec(method)
        if '_labels' in spec.args or spec.keywords:
            kwargs['_labels'] = node.get_labels()
        if '_text' in spec.args or spec.keywords:
            kwargs['_text'] = node.payload
        method(*args, **kwargs)

    def permute_schedule(self, node):
        return node.permute_schedule()


class StepMatcherVisitor(IVisitor):
    """ Visits all steps in order to find missing step methods. """

    def __init__(self, suite, matcher):
        self._suite = suite
        self._not_matched = OrderedDict()
        self._matcher = matcher

    def visit(self, node):
        try:
            node.find_step(self._matcher)
        except MissingStepError as e:
            if e.docstring:
                self._not_matched[e.docstring] = e.suggest
            else:
                self._not_matched[e.method_name] = e.suggest

    def after_visit(self, node):
        pass

    def report_missing(self):
        suggest = u''.join(self._not_matched.values())
        if suggest:
            diagnostic = u'Cannot match steps:\n\n{}'.format(suggest)
            self._suite.fail(to_docstring(diagnostic))



import unittest

from mock import patch, Mock
from morelia.decorators import should_skip, tags


@tags(['unit'])
class ShouldSkipTestCase(unittest.TestCase):

    def test_should_skip(self):
        tags_list = ['tag1', 'tag2']
        test_data = [
            ('', False),
            ('-tag1', True),
            ('tag1', False),
            ('tag1 tag2', False),
            ('tag1 -tag2', True),
            ('-tag1 -tag2', True),
            ('tag3', True),
            ('-tag3', False),
            ('tag1 -tag3', False),
            ('tag1 tag2 -tag3', False),
            ('-tag1 -tag2 -tag3', True),
        ]
        for pattern, expected in test_data:
            result = should_skip(tags_list, pattern)
            self.assertEqual(result, expected)


@tags(['unit'])
class TagsTestCase(unittest.TestCase):

    def setUp(self):
        self._skip_data = [
            ([], 'tag1'),
            (['tag1'], '-tag1'),
        ]
        self._pass_data = [
            (['tag1'], 'tag1'),
            (['tag1'], ''),
            ([], '-tag1'),
        ]

    def dummy(self):
        pass

    def test_should_skip(self):
        for tags_list, pattern in self._skip_data:
            with patch('morelia.decorators.get_config') as get_config:
                get_config.return_value.get_tags_pattern.return_value = pattern
                decorated = tags(tags_list)(self.dummy)
                self.assertRaises(unittest.SkipTest, decorated)

    def test_should_not_skip(self):
        for tags_list, pattern in self._pass_data:
            with patch('morelia.decorators.get_config') as get_config:
                get_config.return_value.get_tags_pattern.return_value = pattern
                decorated = tags(tags_list)(self.dummy)
                try:
                    decorated()
                except unittest.SkipTest:  # pragma: nocover
                    self.fail('Should not raise SkipTest')  # pragma: nocover

    def test_should_skip_with_given_config(self):
        for tags_list, pattern in self._skip_data:
            config = Mock()
            config.get_tags_pattern.return_value = pattern
            decorated = tags(tags_list, config=config)(self.dummy)
            self.assertRaises(unittest.SkipTest, decorated)

    def test_should_not_skip_with_no_pattern_func(self):
        for tags_list, pattern in self._pass_data:
            config = Mock()
            config.get_tags_pattern.return_value = pattern
            decorated = tags(tags_list, config=config)(self.dummy)
            try:
                decorated()
            except unittest.SkipTest:  # pragma: nocover
                self.fail('Should not raise SkipTest')  # pragma: nocover



class MoreliaError(Exception):
    pass


class MissingStepError(MoreliaError):

    def __init__(self, predicate, suggest, method_name, docstring, *args, **kwargs):
        self.args = (suggest, predicate, method_name, docstring)
        self.predicate = predicate
        self.suggest = suggest
        self.method_name = method_name
        self.docstring = docstring



from abc import ABCMeta
import copy
import itertools
import re

from six import moves

from .exceptions import MissingStepError
from .i18n import TRANSLATIONS
from .utils import to_unicode


class INode(object):

    __metaclass__ = ABCMeta

    def find_step(self, matcher):
        """Find method matching step.

        :param IStepMatcher matcher: object matching methods by given predicate
        :returns: (method, args, kwargs) tuple
        :rtype: tuple
        :raises MissingStepError: if method maching step not found
        """
        pass

    def accept(self, visitor):
        """Iterate through grammar tree accepting visitor.

        :param IVisitor visitor: visitor object
        """
        try:
            visitor.visit(self)
            for step in self.steps:
                step.accept(visitor)
        finally:
            visitor.after_visit(self)


class LabeledNode(object):

    def __init__(self, *args, **kwargs):
        super(LabeledNode, self).__init__(*args, **kwargs)
        self._labels = []

    def add_labels(self, tags):
        self._labels.extend(tags)

    def get_labels(self):
        labels = self._labels
        if self.parent:
            labels.extend(self.parent.get_labels())
        return labels


class Morelia(LabeledNode, INode):

    def __init__(self, keyword, predicate, steps=[]):
        super(Morelia, self).__init__()
        self.parent = None
        self.additional_data = {}
        self.keyword = keyword
        self.predicate = predicate if predicate is not None else ''
        self.steps = steps

    def connect_to_parent(self, steps=[], line_number=0):
        self.steps = []
        self.line_number = line_number

        mpt = self.my_parent_type()
        try:
            for step in steps[::-1]:
                if isinstance(step, mpt):
                    step.steps.append(self)
                    self.parent = step
                    break
        except TypeError:
            self.enforce(False, 'Only one Feature per file')

        steps.append(self)
        return self

    def prefix(self):
        return ''

    def my_parent_type(self):
        return None

    @classmethod
    def get_pattern(cls, language):
        class_name = cls.__name__
        name = class_name.lower()
        name = TRANSLATIONS[language].get(name, class_name)
        return r'\s*(?P<keyword>' + name + '):?(?:\s+(?P<predicate>.*))?$'

    def count_dimensions(self):
        ''' Get number of rows. '''
        return sum([step.count_dimension() for step in self.steps])

    def count_dimension(self):
        return 0

    def validate_predicate(self):
        return  # looks good! (-:

    def enforce(self, condition, diagnostic):
        if not condition:
            text = ''
            offset = 1
            if self.parent:
                text = self.parent.reconstruction()
                offset = 5
            text += self.reconstruction()
            text = text.replace('\n\n', '\n').replace('\n', '\n\t')
            raise SyntaxError(diagnostic, (self.get_filename(), self.line_number, offset, text))

    def format_fault(self, diagnostic):
        parent_reconstruction = ''
        if self.parent:
            parent_reconstruction = self.parent.reconstruction().strip('\n')
        reconstruction = self.reconstruction()
        args = (self.get_filename(), self.line_number, parent_reconstruction, reconstruction, diagnostic)
        args = tuple([to_unicode(i) for i in args])
        return u'\n  File "%s", line %s, in %s\n %s\n%s' % args

    def reconstruction(self):
        predicate = to_unicode(self.predicate)
        recon = u'%s%s: %s' % (self.prefix(), self.keyword, predicate)
        recon += '\n' if recon[-1] != '\n' else ''
        return recon

    def get_real_reconstruction(self):
        return self.reconstruction() + '\n'

    def get_filename(self):
        node = self

        while node:
            if not node.parent:
                try:
                    return node.filename
                except AttributeError:
                    pass
            node = node.parent

        return None


class Feature(Morelia):

    def prepend_steps(self, scenario):
        background = self.steps[0]
        try:
            background.prepend_steps(scenario)
        except AttributeError:
            pass

    def reconstruction(self):
        predicate = to_unicode(self.predicate)
        predicate = predicate.replace('\n', '\n    ')
        recon = u'%s%s: %s' % (self.prefix(), self.keyword, predicate)
        recon += '\n' if recon[-1] != '\n' else ''
        return recon


class Scenario(Morelia):

    def my_parent_type(self):
        return Feature

    def accept(self, visitor):
        self.parent.prepend_steps(self)
        self.enforce(0 < len(self.steps), 'Scenario without step(s) - Step, Given, When, Then, And, or #')
        schedule = visitor.permute_schedule(self)

        for indices in schedule:
            self.row_indices = indices
            super(Scenario, self).accept(visitor)

    def permute_schedule(self):
        dims = self.count_Row_dimensions()
        return _permute_indices(dims)

    def count_Row_dimensions(self):
        return [step.count_dimensions() for step in self.steps]

    def reconstruction(self):
        return '\n' + self.keyword + ': ' + self.predicate


class Background(Morelia):

    def my_parent_type(self):
        return Feature

    def accept(self, visitor):
        try:
            visitor.visit(self)
        finally:
            visitor.after_visit(self)

    def prepend_steps(self, scenario):
        try:
            return scenario.background_steps
        except AttributeError:
            background_steps = []
            for step in self.steps:
                new_step = copy.copy(step)
                new_step.parent = scenario
                background_steps.append(new_step)
            scenario.steps = background_steps + scenario.steps
            scenario.background_steps = background_steps
            return background_steps


class Step(Morelia):

    def __init__(self, *args, **kwargs):
        super(Step, self).__init__(*args, **kwargs)
        self.payload = ''

    def prefix(self):
        return '  '

    def my_parent_type(self):
        return (Scenario, Background)

    def find_step(self, matcher):
        """Find method matching step.

        :param IStepMatcher matcher: object matching methods by given predicate
        :returns: (method, args, kwargs) tuple
        :rtype: tuple
        :raises MissingStepError: if method maching step not found
        """
        predicate = self.predicate
        augmented_predicate = self._augment_predicate()
        method, args, kwargs = matcher.find(predicate, augmented_predicate)
        if method:
            return method, args, kwargs

        suggest, method_name, docstring = matcher.suggest(predicate)
        raise MissingStepError(predicate, suggest, method_name, docstring)

    def get_real_reconstruction(self):
        predicate = to_unicode(self._augment_predicate())
        recon = u'    %s %s' % (self.keyword, predicate)
        recon += '\n' if recon[-1] != '\n' else ''
        return recon

    def _augment_predicate(self):
        if self.parent is None:
            return self.predicate
        dims = self.parent.count_Row_dimensions()
        if set(dims) == set([0]):
            return self.predicate
        rep = re.compile(r'\<(\w+)\>')
        replitrons = rep.findall(self.predicate)
        if replitrons == []:
            return self.predicate
        self.copy = self.predicate[:]
        row_indices = self.parent.row_indices

        for replitron in replitrons:
            for x in range(0, len(row_indices)):
                table = self.parent.steps[x].steps

                try:
                    row = next(moves.filter(lambda step: isinstance(step, Row), table))
                except StopIteration:
                    pass
                else:
                    for q, title in enumerate(row.harvest()):
                        self.replace_replitron(x, q, row_indices, table, title, replitron)

        return self.copy

    def replace_replitron(self, x, q, row_indices, table, title, replitron):
        if title != replitron:
            return
        at = row_indices[x] + 1

        assert at < len(table), 'this should never happen'

        stick = table[at].harvest()
        found = stick[q]
        found = found.replace('\n', '\\n')
        self.copy = self.copy.replace('<%s>' % replitron, found)


class Given(Step):

    pass


class When(Step):

    pass


class Then(Step):

    pass


class And(Step):

    pass


class But(And):

    pass


class Row(Morelia):

    @classmethod
    def get_pattern(cls, language):
        return r'\s*(?P<keyword>\|):?\s+(?P<predicate>.*)'

    def my_parent_type(self):
        return (Step, Examples)

    def prefix(self):
        return ' ' * 8

    def reconstruction(self):
        recon = self.prefix() + '| ' + self.predicate
        recon += '\n' if recon[-1] != '\n' else ''
        return recon

    def count_dimension(self):
        if self is self.parent.steps[0]:
            # header row
            return 0
        return 1

    def harvest(self):
        row = re.split(r' \|', re.sub(r'\|$', '', self.predicate))
        row = [s.strip() for s in row]
        return row


class Examples(Morelia):

    def prefix(self):
        return ' ' * 4

    def my_parent_type(self):
        return Scenario


class Comment(Morelia):

    def my_parent_type(self):
        return Morelia  # aka "any"

    @classmethod
    def get_pattern(cls, language):
        return r'\s*(?P<keyword>\#)(?P<predicate>.*)'

    def validate_predicate(self):
        self.enforce(self.predicate.count('\n') == 0, 'linefeed in comment')

    def reconstruction(self):
        recon = '    # ' + self.predicate
        recon += '\n' if recon[-1] != '\n' else ''
        return recon


def _special_range(n):
    return moves.range(n) if n else [0]


def _permute_indices(arr):
    product_args = list(_imap(arr))
    result = list(itertools.product(*product_args))
    return result
    #  tx to Chris Rebert, et al, on the Python newsgroup for curing my brainlock here!!


def _imap(*iterables):
    iterables = [iter(i) for i in iterables]
    while True:
        args = [next(i) for i in iterables]
        yield _special_range(*args)



"""
Formatting output
=================

Morelia complies with Unix's `Rule of Silence` [#ROS]_ so when you hook it like this:

.. code-block:: python

    run(filename, self)

and all tests passes it would say nothing:

.. code-block:: console

    $ python -m unittest test_acceptance
    .
    ----------------------------------------------------------------------
    Ran 1 test in 0.028s

    OK

(here's only information from test runner)

But when something went wrong it would complie with Unix's `Rule of Repair` [#ROR]_
and fail noisily:

.. code-block:: console

    F
    ======================================================================
    FAIL: test_addition (test_acceptance.CalculatorTestCase)
    Addition feature
    ----------------------------------------------------------------------
    Traceback (most recent call last):
      File "test_acceptance.py", line 45, in test_addition
        run(filename, self)
      File "(..)/morelia/__init__.py", line 22, in run
        return ast.evaluate(suite, **kwargs)
      File "(...)/morelia/grammar.py", line 36, in evaluate
        feature.evaluate_steps(test_visitor)
      File "(...)/morelia/grammar.py", line 74, in evaluate_steps
        self._evaluate_child_steps(visitor)
      File "(...)/morelia/grammar.py", line 80, in _evaluate_child_steps
        step.evaluate_steps(visitor)
      File "(...)/morelia/grammar.py", line 226, in evaluate_steps
        self.evaluate_test_case(visitor, step_indices)  # note this works on reports too!
      File "(...)/morelia/grammar.py", line 237, in evaluate_test_case
        step.evaluate_steps(visitor)
      File "(...)/morelia/grammar.py", line 73, in evaluate_steps
        visitor.visit(self)
      File "(...)/morelia/visitors.py", line 53, in visit
        node.test_step(self._suite, self._matcher)
      File "(...)/morelia/grammar.py", line 366, in test_step
        self.evaluate(suite, matcher)
      File "(...)/morelia/grammar.py", line 362, in evaluate
        method(*args, **kwargs)
      File "test_acceptance.py", line 41, in step_the_result_should_be_on_the_screen
        self.assertEqual(int(number), self.calculator.get_result())
    AssertionError:
      File "calculator.feature", line 11, in Scenario: Add two numbers
       Then: the result should be "121" on the screen

    121 != 120

    ----------------------------------------------------------------------
    Ran 1 test in 0.020s

    FAILED (failures=1)

Verbosity
---------

In Behaviour Driven Development participate both programmers and non-programmers
and the latter like animations and so on. So to make Morelia a little more verbose
you can pass a `verbose=True` into :py:func:`morelia.run` method.

.. code-block:: python

    run(filename, self, verbose=True)

.. code-block:: console

    Feature: Addition
        In order to avoid silly mistakes
        As a math idiot
        I want to be told the sum of two numbers
    Scenario: Add two numbers
        Given I have powered calculator on                       # pass  0.000s
        When I enter "50" into the calculator                    # pass  0.000s
        And I enter "70" into the calculator                     # pass  0.000s
        And I press add                                          # pass  0.001s
        Then the result should be "120" on the screen            # pass  0.001s
    Scenario: Subsequent additions
        Given I have powered calculator on                       # pass  0.000s
        When I enter "50" into the calculator                    # pass  0.000s
        And I enter "70" into the calculator                     # pass  0.000s
        And I press add                                          # pass  0.001s
        And I enter "20" into the calculator                     # pass  0.000s
        And I press add                                          # pass  0.001s
        Then the result should be "140" on the screen            # pass  0.001s
    .
    ----------------------------------------------------------------------
    Ran 1 test in 0.027s

    OK

With ``verbose=True`` Morelia tries to use :py:class:`morelia.formatters.ColorTextFormatter` if avaiable in system
and fallbacks to :py:class:`morelia.formatters.PlainTextFormatter` if can't show colors.

You can explicity pass formatter you want use:

.. code-block:: python

    from morelia.formatters import ColorTextFormatter

    run(filename, self, formatter=ColorTextFormatter())

.. code-block:: console

    Feature: Addition
        In order to avoid silly mistakes
        As a math idiot
        I want to be told the sum of two numbers
    Scenario: Add two numbers
        Given I have powered calculator on                       # 0.000s
        When I enter "50" into the calculator                    # 0.000s
        And I enter "70" into the calculator                     # 0.000s
        And I press add                                          # 0.001s
        Then the result should be "120" on the screen            # 0.001s
    Scenario: Subsequent additions
        Given I have powered calculator on                       # 0.000s
        When I enter "50" into the calculator                    # 0.000s
        And I enter "70" into the calculator                     # 0.000s
        And I press add                                          # 0.001s
        And I enter "20" into the calculator                     # 0.000s
        And I press add                                          # 0.001s
        Then the result should be "140" on the screen            # 0.001s
    .
    ----------------------------------------------------------------------
    Ran 1 test in 0.027s

    OK

(You have to run above for yourself to see colors - sorry).

Or you can write your own formatter.

Missing steps
-------------

By default Morelia prints all missing steps if it finds out that some steps
are missing. If you pass ``show_all_missing=False`` then only first missing step
will be shown. It can be usefull when working with features with many steps.

Formatter Classes
-----------------

"""

from abc import ABCMeta, abstractmethod
import sys

from .grammar import Step, Feature

colors = {
    'normal': u'\x1b[30m',
    'fail': u'\x1b[31m',
    'error': u'\x1b[31m',
    'pass': u'\x1b[32m',
    'reset': u'\x1b[0m',
}


class IFormatter(object):

    ''' Abstract Base Class for all formatters. '''

    __metaclass__ = ABCMeta

    @abstractmethod
    def output(self, node, line, status, duration):
        ''' Method called after execution each step.

        :param INode node: node representing step
        :param str line: text of executed step
        :param str status: execution status
        :param float duration: step execution duration
        '''
        pass  # pragma: nocover


class NullFormatter(IFormatter):

    ''' Formatter that... do nothing. '''

    def output(self, node, line, status, duration):
        ''' See :py:meth:`IFormatter.output` '''

        pass


class PlainTextFormatter(IFormatter):

    ''' Formatter that prints all executed steps in plain text to a given stream. '''

    def __init__(self, stream=None):
        ''' Initialize formatter.

        :param file stream: file-like stream to output executed steps
        '''
        self._stream = stream if stream is not None else sys.stderr

    def output(self, node, line, status, duration):
        ''' See :py:meth:`IFormatter.output` '''

        if isinstance(node, Feature):
            self._stream.write('\n')
        if isinstance(node, Step):
            status = status.lower()
            text = '%-60s # %-5s %.3fs\n' % (
                line.strip('\n'),
                status,
                duration,
            )
        else:
            text = '%s\n' % line.strip('\n')
        self._stream.write(text)
        self._stream.flush()


class ColorTextFormatter(PlainTextFormatter):

    ''' Formatter that prints all executed steps in color to a given stream. '''

    def output(self, node, line, status, duration):
        ''' See :py:meth:`IFormatter.output` '''

        if isinstance(node, Feature):
            self._stream.write('\n')
        if isinstance(node, Step):
            status = status.lower()
            text = '%s%-60s # %.3fs%s\n' % (
                colors[status],
                line.strip('\n'),
                duration,
                colors['reset']
            )
        else:
            text = '%s\n' % line.strip('\n')
        self._stream.write(text)
        self._stream.flush()



# -*- coding: utf-8 -*-
import os
from unittest import TestCase

from morelia.decorators import tags
from morelia.parser import Parser

pwd = os.path.dirname(os.path.realpath(__file__))


@tags(['acceptance'])
class DocStringTest(TestCase):

    def test_docstrings_should_be_passed_as_text_to_step(self):
        filename = os.path.join(pwd, 'features/docstrings.feature')
        ast = Parser().parse_file(filename)
        ast.evaluate(self, show_all_missing=True)

    def step_with_docstring(self, _text=None):
        r'step with docstring'

        self.assertIsNotNone(_text)
        self._text = _text

    def step_step_without_docstring(self):
        r'step without docstring'

        self.assertEqual(self._text, 'Docstring line1\nline2')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest import TestCase

from mock import sentinel, Mock, patch

from morelia import run
from morelia.decorators import tags
from morelia.parser import Parser
from morelia.grammar import Morelia, Step
from morelia.exceptions import MissingStepError


@tags(['unit'])
class MoreliaReconstructionTestCase(TestCase):
    """ Test :py:meth:`Morelia.reconstruction`. """

    def test_should_reconstruct_unicode(self):
        """ Scenario: unicode input """
        # Arrange
        obj = Morelia('???', u'zażółć gęślą jaźń')
        # Act
        result = obj.reconstruction()
        # Assert
        self.assertEqual(result, u'???: zażółć gęślą jaźń\n')

    def test_should_reconstruct_utf8(self):
        """ Scenario: utf8 input """
        # Arrange
        obj = Morelia('???', 'zażółć gęślą jaźń')
        # Act
        result = obj.reconstruction()
        # Assert
        self.assertEqual(result, u'???: zażółć gęślą jaźń\n')


@tags(['unit'])
class MissingStepErrorTestCase(TestCase):
    """ Test :py:meth:`MissingStepError`. """

    def test_should_create_exception(self):
        """ Scenario: exception with step and suggest """
        # Arrange
        # Act
        result = MissingStepError('predicate line', 'def step_predicate_line(self):\n    pass', 'step_predicate_line', '')
        # Assert
        self.assertTrue(result is not None)
        self.assertEqual(result.predicate, 'predicate line')


@tags(['unit'])
class StepFindStepTestCase(TestCase):
    """ Test :py:meth:`Step.find_step`. """

    def test_should_find_method(self):
        """ Scenario: method found """
        # Arrange
        obj = Step('???', 'method')
        # Act
        matcher = Mock()
        matcher.find.return_value = (sentinel.method, [], {})
        result, args, kwargs = obj.find_step(matcher)
        # Assert
        self.assertEqual(result, sentinel.method)

    def test_should_not_find_method(self):
        """ Scenario: not found """
        # Arrange
        obj = Step('???', u'some_method')
        matcher = Mock()
        matcher.find.return_value = (None, [], {})
        matcher.suggest.return_value = ('suggest', 'method_name', 'docstring')
        self.assertRaises(MissingStepError, obj.find_step, matcher)


@tags(['unit'])
class RunTestCase(TestCase):
    """ Test :py:func:`run`. """

    @patch('morelia.has_color_support')
    @patch('morelia.PlainTextFormatter')
    @patch.object(Parser, 'parse_file')
    def test_should_parse_file_and_evaluate_tests(self, parse_file, PlainTextFormatter, has_color_support):
        """ Scenario: run and evaluate """
        has_color_support.return_value = False
        run(sentinel.filename, sentinel.suite)
        parse_file.assert_called_once_with(sentinel.filename)
        parse_file.return_value.evaluate.assert_called_once_with(
            sentinel.suite, show_all_missing=True)

    @patch('morelia.has_color_support')
    @patch('morelia.PlainTextFormatter')
    @patch.object(Parser, 'parse_file')
    def test_should_run_verbose_with_plain_text_formatter(self, parse_file, PlainTextFormatter, has_color_support):
        """ Scenario: run verbose on windows """
        has_color_support.return_value = False
        run(sentinel.filename, sentinel.suite, verbose=True)
        parse_file.assert_called_once_with(sentinel.filename)
        parse_file.return_value.evaluate.assert_called_once_with(
            sentinel.suite, formatter=PlainTextFormatter.return_value,
            show_all_missing=True)

    @patch('morelia.has_color_support')
    @patch('morelia.ColorTextFormatter')
    @patch.object(Parser, 'parse_file')
    def test_should_run_verbose_with_color_text_formatter(self, parse_file, ColorTextFormatter, has_color_support):
        """ Scenario: run verbose on systems with color support """
        has_color_support.return_value = True
        run(sentinel.filename, sentinel.suite, verbose=True)
        parse_file.assert_called_once_with(sentinel.filename)
        parse_file.return_value.evaluate.assert_called_once_with(
            sentinel.suite, formatter=ColorTextFormatter.return_value,
            show_all_missing=True)

    @patch.object(Parser, 'parse_file')
    def test_should_use_provided_formatter(self, parse_file):
        """ Scenario: run verbose on systems with color support """
        run(sentinel.filename, sentinel.suite, verbose=True, formatter=sentinel.formatter)
        parse_file.assert_called_once_with(sentinel.filename)
        parse_file.return_value.evaluate.assert_called_once_with(
            sentinel.suite, formatter=sentinel.formatter,
            show_all_missing=True)



# -*- coding: utf-8 -*-
import os
from unittest import TestCase

from morelia.decorators import tags
from morelia.parser import Parser

pwd = os.path.dirname(os.path.realpath(__file__))


@tags(['acceptance'])
class CommentsTest(TestCase):

    def test_comments(self):
        filename = os.path.join(pwd, 'features/comments.feature')
        ast = Parser().parse_file(filename)
        ast.evaluate(self, show_all_missing=True)

    def step_I_have_some_comment_after_step_without_table(self):
        r'I have some comment after step without table'

        pass

    def step_I_have_interpolated_data_from_table(self, data):
        r'I have interpolated (.+) from table'

        pass

    def step_I_execute_this_scenario(self):
        pass

    def step_scenario_will_pass(self):
        assert True



# -*- coding: utf-8 -*-
#            __  __                _ _
#           |  \/  | ___  _ __ ___| (_) __ _
#           | |\/| |/ _ \| '__/ _ \ | |/ _` |
#           | |  | | (_) | | |  __/ | | (_| |
#           |_|  |_|\___/|_|  \___|_|_|\__,_|
#                             o        o     |  o
#                                 ,_       __|      ,
#                        |  |_|  /  |  |  /  |  |  / \_
#                         \/  |_/   |_/|_/\_/|_/|_/ \/
import re
import textwrap


from .formatters import NullFormatter
from .grammar import (Feature, Background, Scenario, Given, When, Then,
                      And, But, Row, Comment, Examples, Step)
from .matchers import RegexpStepMatcher, ParseStepMatcher, MethodNameStepMatcher
from .visitors import TestVisitor, StepMatcherVisitor
from .i18n import TRANSLATIONS
from .utils import fix_exception_encoding, to_unicode


class AST(object):

    def __init__(self, steps, test_visitor_class=TestVisitor,
                 matcher_visitor_class=StepMatcherVisitor):
        self.steps = steps
        self._test_visitor_class = test_visitor_class
        self._matcher_visitor_class = matcher_visitor_class

    def evaluate(self, suite, formatter=None, matchers=None, show_all_missing=True):
        if matchers is None:
            matchers = [RegexpStepMatcher, ParseStepMatcher, MethodNameStepMatcher]
        matcher = self._create_matchers_chain(suite, matchers)
        feature = self.steps[0]
        if show_all_missing:
            matcher_visitor = self._matcher_visitor_class(suite, matcher)
            feature.accept(matcher_visitor)
            matcher_visitor.report_missing()
        if formatter is None:
            formatter = NullFormatter()
        test_visitor = self._test_visitor_class(suite, matcher, formatter)
        try:
            feature.accept(test_visitor)
        except SyntaxError as exc:
            raise
        except Exception as exc:
            fix_exception_encoding(exc)
            raise

    def _create_matchers_chain(self, suite, matcher_classes):
        root_matcher = None
        for matcher_class in matcher_classes:
            matcher = matcher_class(suite)
            try:
                root_matcher.add_matcher(matcher)
            except AttributeError:
                root_matcher = matcher
        return root_matcher


class Parser(object):

    def __init__(self, language=None):
        self.thangs = [
            Feature, Background, Scenario,
            Given, When, Then, And, But,
            Row, Comment, Examples, Step
        ]
        self.steps = []
        if language is None:
            language = 'en'
        self._language = language
        self._prepare_patterns(language)

    def _prepare_patterns(self, language):
        self._patterns = []
        for thang in self.thangs:
            pattern = thang.get_pattern(language)
            self._patterns.append((re.compile(pattern), thang))

    def parse_file(self, filename):
        with open(filename, 'rb') as input_file:
            prose = input_file.read().decode('utf-8')
            ast = self.parse_features(prose)
            self.steps[0].filename = filename
            return ast

    def parse_features(self, prose):
        self.parse_feature(prose)
        ast = AST(self.steps)
        feature = self.steps[0]
        assert isinstance(feature, Feature), 'Exactly one Feature perf file'
        feature.enforce(any(isinstance(step, Scenario) for step in feature.steps), 'Feature without Scenario(s)')
        return ast

    def _parse_line(self, line):
        if self._language_parser.parse(line):
            self._prepare_patterns(self._language_parser.language)
            return

        if self._labels_parser.parse(line):
            return

        if self._docstring_parser.parse(line):
            previous = self.steps[-1]
            previous.payload = self._docstring_parser.payload
            return

        if self._anneal_last_broken_line(line):
            return

        if self._parse_thang(line):
            return

        if 0 < len(self.steps):
            self._append_to_previous_node(line)
        else:
            s = Step('???', line)
            s.line_number = self._line_producer.line_number
            feature_name = TRANSLATIONS[self._language_parser.language].get('feature', u'Feature')
            feature_name = feature_name.replace('|', ' or ')
            s.enforce(False, u'feature files must start with a %s' % feature_name)

    def parse_feature(self, lines):
        lines = to_unicode(lines)

        self._line_producer = LineSource(lines)
        self._docstring_parser = DocStringParser(self._line_producer)
        self._language_parser = LanguageParser(default_language=self._language)
        self._labels_parser = LabelParser()
        try:
            while True:
                line = self._line_producer.get_line()
                if line:
                    self._parse_line(line)
        except StopIteration:
            pass
        return self.steps

    def _anneal_last_broken_line(self, line):
        if self.steps == []:
            return False
        last_line = self.last_node.predicate

        if re.search(r'\\\s*$', last_line):
            last = self.last_node
            last.predicate += '\n' + line
            return True

        return False

    def _parse_thang(self, line):
        line = line.rstrip()

        for regexp, klass in self._patterns:
            m = regexp.match(line)

            if m and len(m.groups()) > 0:
                node = klass(**m.groupdict())
                node.add_labels(self._labels_parser.pop_labels())
                node.connect_to_parent(self.steps, self._line_producer.line_number)
                self.last_node = node
                return node

    def _append_to_previous_node(self, line):
        previous = self.steps[-1]
        previous.predicate += '\n' + line.strip()
        previous.predicate = previous.predicate.strip()
        previous.validate_predicate()


class LabelParser(object):

    def __init__(self, labels_pattern='@\w+'):
        self._labels = []
        self._labels_re = re.compile(labels_pattern)
        self._labels_prefix_re = re.compile('^\s*@')

    def parse(self, line):
        """ Parse labels.

        :param str line: line to parse
        :returns: True if line contains labels
        :side effects: sets self._labels to parsed labels
        """
        if self._labels_prefix_re.match(line):
            matches = self._labels_re.findall(line)
            if matches:
                self._labels.extend(matches)
                return True
        return False

    def pop_labels(self):
        """ Return labels.

        :returns: labels
        :side effects: clears current labels
        """
        labels = [label.strip('@') for label in self._labels]
        self._labels = []
        return labels


class LanguageParser(object):

    def __init__(self, lang_pattern='^# language: (\w+)', default_language=None):
        if default_language is None:
            default_language = 'en'
        self._language = default_language
        self._lang_re = re.compile(lang_pattern)

    def parse(self, line):
        """ Parse language directive.

        :param str line: line to parse
        :returns: True if line contains language directive
        :side effects: sets self.language to parsed language
        """
        match = self._lang_re.match(line)
        if match:
            self._language = match.groups()[0]
            return True
        return False

    @property
    def language(self):
        return self._language


class DocStringParser(object):

    def __init__(self, source, pattern='\s*"""\s*'):
        self._source = source
        self._docstring_re = re.compile(pattern)
        self._payload = []

    def parse(self, line):
        """ Parse docstring payload.

        :param str line: first line to parse
        :returns: True if docstring parsed
        :side effects: sets self.payload to parsed docstring
        """
        match = self._docstring_re.match(line)
        if match:
            start_line = line
            self._payload = []
            line = self._source.get_line()
            while line != start_line:
                self._payload.append(line)
                line = self._source.get_line()
            return True
        return False

    @property
    def payload(self):
        return textwrap.dedent('\n'.join(self._payload))


class LineSource(object):

    def __init__(self, text):
        self._lines = iter([line for line in text.split('\n') if line])
        self._line_number = 0

    def get_line(self):
        """ Return next line.

        :returns: next line of text
        """
        self._line_number += 1
        return next(self._lines)

    @property
    def line_number(self):
        return self._line_number



# -*- coding: utf-8 -*-
"""
Running
=======

To run scenarios Morelia's :py:func:`run` method needs to be given at least
two parameters:

    * file name with scenarios description
    * TestCase with defined steps

Then running is as simple as:

.. code-block:: python

    run('calculator.feature', test_case_with_steps)

"""

import sys

from .formatters import PlainTextFormatter, ColorTextFormatter
from .parser import Parser  # noqa

__version__ = '0.6.5'


def has_color_support():
    """Check if color in terminal is supported."""

    return sys.platform != 'win32'  # pragma: nocover


def run(filename, suite, verbose=False, show_all_missing=True, **kwargs):
    """ Parse file and run tests on given suite.

    :param str filename: file name
    :param unittest.TestCase suite: TestCase instance
    :param boolean verbose: be verbose
    :param boolean show_all_missing: show all missing steps
    """
    formatter = kwargs.get('formatter', None)
    if verbose and not formatter:
        if has_color_support():
            formatter = ColorTextFormatter()
        else:
            formatter = PlainTextFormatter()
        kwargs['formatter'] = formatter
    kwargs['show_all_missing'] = show_all_missing
    parser = Parser()
    ast = parser.parse_file(filename)
    return ast.evaluate(suite, **kwargs)


__all__ = ('Parser', 'run')



"""
Configuration
-------------
"""

import os

from six.moves.configparser import (
    SafeConfigParser,
    NoSectionError,
    NoOptionError
)


DEFAULT_CONFIG_FILES = ['.moreliarc', '~/.moreliarc', '/etc/morelia.rc']


def expand_all(path):
    """Expand path."""
    return os.path.abspath(os.path.expandvars(os.path.expanduser(path)))


class Config(object):

    """ Configuration object.

    Configuration is read from ini-style files and environment variables
    prefixed with `MORELIA_`.
    By default Morelia search for files:

        * .moreliarc
        * ~/.moreliarc
        * /etc/morelia.rc
    """

    def __init__(self, config_files=None, config_parser_class=None):
        self._env_prefix = 'MORELIA_'
        self._items = {
            'tags': None,
            'formatter': None,
            'matchers': None,
            'show_all_missing': False,
        }
        if config_files is None:
            config_files = DEFAULT_CONFIG_FILES
        self._default_section = 'morelia'
        self._config_files = [expand_all(config_file)
                              for config_file in config_files]
        if config_parser_class is None:
            config_parser_class = SafeConfigParser
        self._config_parser_class = config_parser_class

    def load(self):
        """Load configuration."""

        self._update_from_file()
        self._update_from_environ()

    def _update_from_file(self):
        """Update config on settings from *.ini file."""

        config_parser = self._config_parser_class()
        config_parser.read(self._config_files)
        for key in self._items.keys():
            try:
                value = config_parser.get('morelia', key)
            except (NoOptionError, NoSectionError):
                pass
            else:
                self._items[key] = value

    def _update_from_environ(self):
        """Update config on environment variables."""

        for key in self._items.keys():
            try:
                value = os.environ[self._env_prefix + key.upper()]
            except KeyError:
                pass
            else:
                self._items[key] = value

    def get_tags_pattern(self):
        """Return tags pattern."""

        tags = self._items.get('tags', '')
        return tags if tags is not None else ''


def get_config(_memo={}):
    """Return config object."""

    try:
        return _memo['config']
    except KeyError:
        config = Config()
        config.load()
        _memo['config'] = config
    return _memo['config']



# -*- coding: utf-8 -*-
import os
from unittest import TestCase

from morelia.decorators import tags
from morelia.parser import Parser
from morelia.grammar import Scenario

pwd = os.path.dirname(os.path.realpath(__file__))


@tags(['acceptance'])
class BackgroundTest(TestCase):

    def test_background(self):
        filename = os.path.join(pwd, 'features/background.feature')
        ast = Parser().parse_file(filename)
        self._scenarios_num = sum(1 for s in ast.steps if isinstance(s, Scenario))
        ast.evaluate(self)

    def step_step_ran_is_number(self, number):
        r'step_ran was "([^"]+)"'

        self._step_ran = int(number)

    def step_alt_step_ran_is_number(self, number):
        r'alt_step_ran was "([^"]+)"'

        self._alt_step_ran = int(number)

    def step_I_increment_step_ran_by_number(self, number):
        r'I increment step_ran by "([^"]+)"'

        self._step_ran += int(number)

    def step_increment_alt_step_ran_by_number(self, number):
        r'increment alt_step_ran by "([^"]+)"'

        self._alt_step_ran += int(number)

    def step_step_ran_equals_number(self, number):
        r'step_ran will equal "([^"]+)"'

        self.assertEqual(self._step_ran, int(number))

    def step_alt_step_ran_equals_number(self, number):
        r'alt_step_ran will equal "([^"]+)"'

        self.assertEqual(self._alt_step_ran, int(number))

    def step_angles_step_is_background(self, background):
        r'angles_step was (.+)'

        self._background = background

    def step_I_increment_angles_step_by_when(self, when):
        r'I increment angles_step by (.+)'

        self._background = int(self._background) + int(when)

    def step_angles_step_equals_then(self, then):
        r'angles_step will equal (.+)'

        self.assertEqual(self._background, int(then))

    def step_angles_step_will_be_then(self, then):
        r'angles_step will be string "([^"]+)"'

        self.assertEqual(self._background, then)

    def step_background_only_step_ran_incremented(self):
        r'background_only_step_ran incremented'

        try:
            self._background_only_step_ran += 1
        except AttributeError:
            self._background_only_step_ran = 1

    def step_background_only_step_ran_will_equal_scenarios_number(self):
        r'background_only_step_ran will equal scenarios number'

        self.assertEqual(self._background_only_step_ran, self._scenarios_num)



# -*- coding: utf-8 -*-
"""
Decorators
----------

Sometimes you need selectively run tests. For that reason you can tag your tests:

.. code-block:: python

    # test_acceptance.py

    import unittest

    from morelia import run
    from morelia.decorators import tags


    class CalculatorTestCase(unittest.TestCase):

        @tags(['basic'])
        def test_addition(self):
            ''' Addition feature '''
            filename = os.path.join(os.path.dirname(__file__), 'add.feature')
            run(filename, self)
            # ...

        @tags(['advanced'])
        def test_substraction(self):
            ''' Substraction feature '''
            filename = os.path.join(os.path.dirname(__file__), 'substract.feature')
            run(filename, self)
            # ...

        @tags(['slow', 'advanced'])
        def test_multiplication(self):
            ''' Multiplication feature '''
            filename = os.path.join(os.path.dirname(__file__), 'multiplication.feature')
            run(filename, self)
            # ...

And run tests only for selected features:

.. code-block:: console

    $ MORELIA_TAGS=basic python -m unittest test_acceptance

    .ss
    ----------------------------------------------------------------------
    Ran 3 test in 0.018s

    OK (skipped=2)

    $ MORELIA_TAGS=advanced python -m unittest test_acceptance

    s..
    ----------------------------------------------------------------------
    Ran 3 test in 0.048s

    OK (skipped=2)

    $ MORELIA_TAGS=-slow python -m unittest test_acceptance

    ..s
    ----------------------------------------------------------------------
    Ran 3 test in 0.028s

    OK (skipped=1)

    $ MORELIA_TAGS=advanced,-slow python -m unittest test_acceptance

    s.s
    ----------------------------------------------------------------------
    Ran 3 test in 0.022s

    OK (skipped=2)
"""

import unittest

from morelia.config import get_config


def should_skip(tags_list, pattern):
    tags_list = set(tags_list)
    matching_tags = pattern.split()
    negative_tags = [tag[1:] for tag in matching_tags if tag.startswith('-')]
    positive_tags = [tag for tag in matching_tags if not tag.startswith('-')]
    if negative_tags:
        return bool(set(negative_tags) & tags_list)
    if positive_tags:
        return not set(positive_tags).issubset(tags_list)
    return False


def tags(tags_list, config=None):
    """ Skip decorated test methods or classes if tags matches.

    Tags are matched to patterns provided by config object.

    :param list tags_list: list of tags for test
    :param morelia.config.Config config: optional configuration object
    """
    if config is None:
        config = get_config()
    pattern = config.get_tags_pattern()
    return unittest.skipIf(should_skip(tags_list, pattern), 'Tags not matched')



import unittest

from mock import Mock, patch
from morelia.config import Config, get_config
from morelia.decorators import tags
from six.moves.configparser import NoSectionError, NoOptionError, SafeConfigParser


@tags(['unit'])
class ConfigGetTagsPatternTestCase(unittest.TestCase):
    """ Test :py:meth:`Config.get_tags_pattern`. """

    @patch('morelia.config.os')
    def test_should_return_tags_pattern_from_file(self, os):
        """ Scenario: tags pattern from file """
        # Arrange
        os.environ.__getitem__.side_effect = KeyError

        def config_read(section, key):
            assert section == 'morelia'
            if key == 'tags':
                return 'tag1,tag2'
            return ''
        config_parser_class = Mock()
        config_parser_class.return_value.get.side_effect = config_read
        obj = Config(config_parser_class=config_parser_class)
        obj.load()
        # Act
        pattern = obj.get_tags_pattern()
        # Assert
        self.assertEqual(pattern, 'tag1,tag2')

    @patch('morelia.config.os')
    def test_should_not_tags_pattern_from_file_if_no_option(self, os):
        """ Scenario: tags pattern from file """
        # Arrange
        os.environ.__getitem__.side_effect = KeyError

        def config_read(section, key):
            assert section == 'morelia'
            if key == 'tags':
                raise NoOptionError(None, None)
            return ''
        config_parser_class = Mock()
        config_parser_class.return_value.get.side_effect = config_read
        obj = Config(config_parser_class=config_parser_class)
        obj.load()
        # Act
        pattern = obj.get_tags_pattern()
        # Assert
        self.assertEqual(pattern, '')

    @patch('morelia.config.os')
    def test_should_not_tags_pattern_from_file_if_no_section(self, os):
        """ Scenario: tags pattern from file """
        # Arrange
        os.environ.__getitem__.side_effect = KeyError

        def config_read(section, key):
            assert section == 'morelia'
            if key == 'tags':
                raise NoSectionError(None)
            return ''
        config_parser_class = Mock()
        config_parser_class.return_value.get.side_effect = config_read
        obj = Config(config_parser_class=config_parser_class)
        obj.load()
        # Act
        pattern = obj.get_tags_pattern()
        # Assert
        self.assertEqual(pattern, '')

    @patch('morelia.config.os')
    def test_should_return_tags_pattern_from_environment(self, os):
        """ Scenario: tags pattern from environment """
        # Arrange

        def environ_get(key):
            if key == 'MORELIA_TAGS':
                return 'tag1,tag2'
            return ''
        os.environ.__getitem__.side_effect = environ_get
        config_parser_class = Mock()
        config_parser_class.return_value.get.return_value = ''
        obj = Config(config_parser_class=config_parser_class)
        obj.load()
        # Act
        pattern = obj.get_tags_pattern()
        # Assert
        self.assertEqual(pattern, 'tag1,tag2')


@tags(['unit'])
class ConfigInitTestCase(unittest.TestCase):
    """ Test :py:meth:`Config.__init__`. """

    def test_should_create_with_config_files(self):
        """ Scenario: create with config files"""
        # Arrange
        # Act
        obj = Config(config_files=['sample.cfg'])
        # Assert
        self.assertEqual(len(obj._config_files), 1)

    def test_should_create_with_default_parser(self):
        """ Scenario: create with default parser """
        # Arrange
        # Act
        obj = Config()
        # Assert
        self.assertTrue(issubclass(SafeConfigParser, obj._config_parser_class))


@tags(['unit'])
class GetConfigTestCase(unittest.TestCase):
    """ Test :py:meth:`get_config`. """

    @patch('morelia.config.Config')
    def test_should_return_config_object(self, Config):
        """ Scenario: return config object """
        # Arrange
        # Act
        config = get_config()
        # Assert
        self.assertIsNotNone(config)

    @patch('morelia.config.Config')
    def test_should_return_memoized_config_object(self, Config):
        """ Scenario: return memoized config object """
        # Arrange
        # Act
        config1 = get_config()
        config2 = get_config()
        # Assert
        self.assertIsNotNone(config1)
        self.assertEqual(config1, config2)



from unittest import TestCase

from mock import sentinel, Mock, ANY

from morelia.decorators import tags
from morelia.formatters import IFormatter
from morelia.grammar import Feature, Morelia, Step
from morelia.matchers import IStepMatcher
from morelia.parser import AST
from morelia.visitors import IVisitor


@tags(['unit'])
class ASTEvaluateTestCase(TestCase):
    """ Test :py:meth:`AST.evaluate`. """

    def test_should_use_provided_matcher(self):
        """ Scenariusz: matcher given as parameter """
        # Arrange
        test_visitor_class = Mock(IVisitor)
        matcher_class = Mock(IStepMatcher)
        feature = Mock(Feature)
        steps = [feature]
        obj = AST(steps, test_visitor_class=test_visitor_class)
        # Act
        obj.evaluate(sentinel.suite, matchers=[matcher_class])
        # Assert
        test_visitor_class.assert_called_once_with(sentinel.suite, matcher_class.return_value, ANY)

    def test_should_use_provided_formatter(self):
        """ Scenariusz: formatter given as parameter """
        # Arrange
        test_visitor = Mock(IVisitor)
        formatter = Mock(IFormatter)
        feature = Mock(Feature)
        steps = [feature]
        obj = AST(steps, test_visitor_class=test_visitor)
        # Act
        obj.evaluate(sentinel.suite, formatter=formatter)
        # Assert
        test_visitor.assert_called_once_with(sentinel.suite, ANY, formatter)

    def test_should_show_all_missing_steps(self):
        """ Scenariusz: show all missing steps """
        # Arrange
        matcher_visitor = Mock(IVisitor)
        feature = Mock(Feature)
        steps = [feature]
        obj = AST(steps, matcher_visitor_class=matcher_visitor)
        suite = Mock()
        # Act
        obj.evaluate(suite, show_all_missing=True)
        # Assert
        matcher_visitor.assert_called_once_with(suite, ANY)


@tags(['unit'])
class LabeledNodeGetLabelsTestCase(TestCase):

    def test_should_return_node_labels(self):
        """ Scenario: node labels """
        # Arrange
        expected = ['label1', 'label2']
        obj = Feature(None, None)
        obj.add_labels(expected)
        # Act
        result = obj.get_labels()
        # Assert
        self.assertEqual(result, expected)

    def test_should_return_node_and_parent_labels(self):
        """ Scenario: node and parent labels """
        # Arrange
        expected = ['label1', 'label2']
        obj = Feature(None, None)
        obj.add_labels(['label1'])
        parent = Feature(None, None)
        parent.add_labels(['label2'])
        obj.parent = parent
        # Act
        result = obj.get_labels()
        # Assert
        self.assertEqual(result, expected)


@tags(['unit'])
class INodeEvaluateStepsTest(TestCase):

    node_class = Morelia

    def test_should_call_visitor(self):
        # Arrange
        node = self.node_class('Feature', 'Some feature')
        visitor = Mock(IVisitor)
        # Act
        node.accept(visitor)
        # Assert
        visitor.visit.assert_called_once_with(node)

    def test_should_evaluate_child_steps(self):
        # Arrange
        steps = [Mock(Step), Mock(Step)]
        node = self.node_class('Feature', 'Some feature', steps=steps)
        visitor = Mock(IVisitor)
        # Act
        node.accept(visitor)
        # Assert
        visitor.visit.assert_called_once_with(node)
        for step in steps:
            step.accept.assert_called_once_with(visitor)



# -*- coding: utf-8 -*-
import os
from unittest import TestCase

from morelia.decorators import tags
from morelia.parser import Parser

pwd = os.path.dirname(os.path.realpath(__file__))


@tags(['acceptance'])
class LabelTest(TestCase):

    def test_labels(self):
        filename = os.path.join(pwd, 'features/labels.feature')
        ast = Parser().parse_file(filename)
        ast.evaluate(self, show_all_missing=True)

    def step_with_labels(self, _labels=None):
        r'step with _labels'

        self.assertIsNotNone(_labels)
        self.assertEqual(set(['label1', 'label2']), set(_labels))

    def step_with_kwargs(self, **kwargs):
        r'step with kwargs'

        _labels = kwargs['_labels']
        self.assertIsNotNone(_labels)

    def step_without_labels(self):
        r'step without _labels'
        pass

    def step_I_should_get_labels(self, labels, _labels):
        r'I should get labels "([^"]+)"'

        expected = ['label{}'.format(label) for label in labels.split(',')]
        self.assertEqual(set(expected), set(_labels))

    def step_step_with_label(self, label, _labels):
        r'step with "([^"]+)"'

        self.assertFalse(label in set(_labels))



from six.moves import StringIO
import unittest

from mock import sentinel, Mock

from morelia.decorators import tags
from morelia.grammar import Step, Scenario, Feature
from morelia.formatters import NullFormatter, PlainTextFormatter, ColorTextFormatter


@tags(['unit'])
class NullFormatterOutputTestCase(unittest.TestCase):
    """ Test :py:meth:`NullFormatter.output`. """

    def test_should_do_nothing(self):
        """ Scenario: do nothing """
        # Arrange
        obj = NullFormatter()
        # Act
        line = 'Given some number'
        status = 'pass'
        duration = 0.01
        obj.output(sentinel.node, line, status, duration)


@tags(['unit'])
class PlainTextFormatterOutputTestCase(unittest.TestCase):
    """ Test :py:meth:`PlainTextFormatter.output`. """

    def test_should_output_for_pass(self):
        """ Scenariusz: format for pass """
        # Arrange
        stream = StringIO()
        obj = PlainTextFormatter(stream)
        # Act
        line = 'Given some number'
        status = 'pass'
        duration = 0.01
        node = Mock(Step)
        obj.output(node, line, status, duration)
        # Assert
        expected = '%-60s # pass  0.010s\n' % line
        self.assertEqual(stream.getvalue(), expected)

    def test_should_output_for_fail(self):
        """ Scenariusz: format for fail """
        # Arrange
        stream = StringIO()
        obj = PlainTextFormatter(stream)
        # Act
        line = 'Given some number'
        status = 'fail'
        duration = 0.01
        node = Mock(Step)
        obj.output(node, line, status, duration)
        # Assert
        expected = '%-60s # fail  0.010s\n' % line
        self.assertEqual(stream.getvalue(), expected)

    def test_should_output_for_error(self):
        """ Scenariusz: format for error """
        # Arrange
        stream = StringIO()
        obj = PlainTextFormatter(stream)
        # Act
        line = 'Given some number'
        status = 'error'
        duration = 0.01
        node = Mock(Step)
        obj.output(node, line, status, duration)
        # Assert
        expected = '%-60s # error 0.010s\n' % line
        self.assertEqual(stream.getvalue(), expected)

    def test_should_output_for_not_step(self):
        """ Scenariusz: format for not a step """
        # Arrange
        stream = StringIO()
        obj = PlainTextFormatter(stream)
        # Act
        line = 'Scenario: some number'
        status = 'pass'
        duration = 0.01
        node = Mock(Scenario)
        obj.output(node, line, status, duration)
        # Assert
        expected = '%s\n' % line
        self.assertEqual(stream.getvalue(), expected)

    def test_should_output_for_feature(self):
        """ Scenariusz: format for feature """
        # Arrange
        stream = StringIO()
        obj = PlainTextFormatter(stream)
        # Act
        line = 'Feature: some feature'
        status = 'pass'
        duration = 0.01
        node = Mock(Feature)
        obj.output(node, line, status, duration)
        # Assert
        expected = '\n%s\n' % line
        self.assertEqual(stream.getvalue(), expected)


@tags(['unit'])
class ColorTextFormatterOutputTestCase(unittest.TestCase):
    """ Test :py:meth:`ColorTextFormatter.output`. """

    def test_should_output_for_pass(self):
        """ Scenariusz: format for pass """
        # Arrange
        stream = StringIO()
        obj = ColorTextFormatter(stream)
        # Act
        line = 'Given some number'
        status = 'pass'
        duration = 0.01
        node = Mock(Step)
        obj.output(node, line, status, duration)
        # Assert
        green = u'\x1b[32m'
        reset = u'\x1b[0m'
        expected = '%s%-60s # 0.010s%s\n' % (green, line, reset)
        self.assertEqual(stream.getvalue(), expected)

    def test_should_output_for_fail(self):
        """ Scenariusz: format for fail """
        # Arrange
        stream = StringIO()
        obj = ColorTextFormatter(stream)
        # Act
        line = 'Given some number'
        status = 'fail'
        duration = 0.01
        node = Mock(Step)
        obj.output(node, line, status, duration)
        # Assert
        red = u'\x1b[31m'
        reset = u'\x1b[0m'
        expected = '%s%-60s # 0.010s%s\n' % (red, line, reset)
        self.assertEqual(stream.getvalue(), expected)

    def test_should_output_for_error(self):
        """ Scenariusz: format for error """
        # Arrange
        stream = StringIO()
        obj = ColorTextFormatter(stream)
        # Act
        line = 'Given some number'
        status = 'error'
        duration = 0.01
        node = Mock(Step)
        obj.output(node, line, status, duration)
        # Assert
        red = u'\x1b[31m'
        reset = u'\x1b[0m'
        expected = '%s%-60s # 0.010s%s\n' % (red, line, reset)
        self.assertEqual(stream.getvalue(), expected)

    def test_should_output_for_not_step(self):
        """ Scenariusz: format for not a step """
        # Arrange
        stream = StringIO()
        obj = ColorTextFormatter(stream)
        # Act
        line = 'Scenario: some number'
        status = 'pass'
        duration = 0.01
        node = Mock(Scenario)
        obj.output(node, line, status, duration)
        # Assert
        expected = '%s\n' % line
        self.assertEqual(stream.getvalue(), expected)

    def test_should_output_for_feature(self):
        """ Scenariusz: format for feature """
        # Arrange
        stream = StringIO()
        obj = ColorTextFormatter(stream)
        # Act
        line = 'Feature: some feature'
        status = 'pass'
        duration = 0.01
        node = Mock(Feature)
        obj.output(node, line, status, duration)
        # Assert
        expected = '\n%s\n' % line
        self.assertEqual(stream.getvalue(), expected)



from unittest import TestCase

from morelia.decorators import tags
from morelia.parser import LabelParser, LanguageParser, LineSource, DocStringParser


@tags(['unit'])
class LabelParserParseTestCase(TestCase):
    """ Test :py:meth:`LabelParser.parse`. """

    def test_should_return_false_if_line_without_labels(self):
        """ Scenario: line without labels """
        # Arrange
        obj = LabelParser()
        line = ''
        # Act
        result = obj.parse(line)
        # Assert
        self.assertFalse(result)

    def test_should_return_true_if_line_with_labels(self):
        """ Scenario: line with labels """
        # Arrange
        obj = LabelParser()
        line = '@label1'
        # Act
        result = obj.parse(line)
        # Assert
        self.assertTrue(result)


@tags(['unit'])
class LabelParserPopLabelsTestCase(TestCase):
    """ Test :py:meth:`LabelParser.pop_labels`. """

    def test_should_not_return_labels(self):
        """ Scenario: no labels """
        # Arrange
        obj = LabelParser()
        # Act
        result = obj.pop_labels()
        # Assert
        self.assertEqual(result, [])

    def test_should_return_labels(self):
        """ Scenario: pop labels """
        # Arrange
        obj = LabelParser()
        obj.parse('@label1')
        # Act
        result = obj.pop_labels()
        # Assert
        self.assertEqual(result, ['label1'])
        self.assertEqual(obj._labels, [])

    def test_should_not_return_labels_when_tag_inside_step(self):
        """ Scenario: no labels when tag inside step"""
        # Arrange
        obj = LabelParser()
        obj.parse('When abc@example.com')
        # Act
        result = obj.pop_labels()
        # Assert
        self.assertEqual(result, [])
        self.assertEqual(obj._labels, [])


@tags(['unit'])
class LanguageParserParseTestCase(TestCase):
    """ Test :py:meth:`LanguageParser.parse`. """

    def test_should_return_false_if_line_without_language_directive(self):
        """ Scenario: line without language directive """
        # Arrange
        obj = LanguageParser()
        lines = [
            '# language: ',  # missing language
            'Feature:',  # not a language directive
            'language: pl',  # missing comment
            '# comment',  # comment
        ]
        for line in lines:
            # Act
            result = obj.parse(line)
            # Assert
            self.assertFalse(result)
            self.assertEqual(obj.language, 'en')

    def test_should_return_true_if_line_with_language_directive(self):
        """ Scenario: line with language directive """
        # Arrange
        obj = LanguageParser()
        line = '# language: pl'
        # Act
        result = obj.parse(line)
        # Assert
        self.assertTrue(result)
        self.assertEqual(obj.language, 'pl')

    def test_should_set_default_language(self):
        """ Scenario: default language passed """
        # Arrange
        obj = LanguageParser(default_language='ja')
        line = ''
        # Act
        result = obj.parse(line)
        # Assert
        self.assertFalse(result)
        self.assertEqual(obj.language, 'ja')


@tags(['unit'])
class LineProducerGetLineTestCase(TestCase):
    """ Test :py:meth:`LineProducer.get_line`. """

    def test_should_raise_stop_iteration_if_no_text(self):
        """ Scenario: no text """
        # Arrange
        obj = LineSource('')
        # Act
        # Assert
        self.assertRaises(StopIteration, obj.get_line)

    def test_should_return_line_by_line(self):
        """ Scenario: return lines """
        # Arrange
        lines = 'line1\nline2\nline3'
        obj = LineSource(lines)
        # Act
        for i, line in enumerate(lines.split('\n')):
            result = obj.get_line()
            # Assert
            self.assertEqual(result, line)
            self.assertEqual(obj.line_number, i + 1)


class DocStringParserParseTestCase(TestCase):
    """ Test :py:meth:`DocStringParser.parse`. """

    def test_should_return_false_if_not_docstring_beginning(self):
        """ Scenario: not a docstring """
        # Arrange
        obj = DocStringParser(None)
        lines = [
            '',
            'Feature: feature',
            '# comment',
            '"',
        ]
        for line in lines:
            # Act
            result = obj.parse(line)
            # Assert
            self.assertFalse(result)

    def test_should_return_true_if_docstring(self):
        """ Scenario: docstring """
        # Arrange
        lines = '"""\nline1\nline2\n"""'
        line_source = LineSource(lines)
        obj = DocStringParser(line_source)
        line = line_source.get_line()
        # Act
        result = obj.parse(line)
        # Assert
        self.assertTrue(result)
        self.assertEqual(obj.payload, 'line1\nline2')

    def test_should_return_true_if_indented_docstring(self):
        """ Scenario: indented docstring """
        # Arrange
        test_data = [
            ('   """\n   line1\n   line2\n   """', 'line1\nline2'),
            ('   """\n     line1\n   line2\n   """', '  line1\nline2'),
        ]
        for lines, expected in test_data:
            line_source = LineSource(lines)
            obj = DocStringParser(line_source)
            line = line_source.get_line()
            # Act
            result = obj.parse(line)
            # Assert
            self.assertTrue(result)
            self.assertEqual(obj.payload, expected)



from setuptools import setup, find_packages

with open('README.rst') as file:
    long_description = file.read()

setup(
    name='Morelia',
    version='0.6.5',
    description='for "Behavior Driven Development" (BDD) -- a client-facing scripting language to put the squeeze on all your features',
    long_description=long_description,
    url='http://morelia.readthedocs.org/',
    author='Phlip, Jakub STOLARSKI (Dryobates)',
    author_email='phlip2005@gmail.com, jakub.stolarski@kidosoft.pl',
    license='MIT',
    keywords='test bdd behavior',
    packages=find_packages('src'),  # , exclude=['example*', '*.tests', 'tests.*', '*.tests.*', 'tests']),
    package_dir={'': 'src'},
    install_requires=[
        'six',
        'parse',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite="morelia.tests",
)



"""
Steps
=====

.. _matching-steps:

Matching steps
--------------

When Morelia executes steps described in feature files it looks
inside passed :py:class:`unittest.TestCase` object and search for methods
which name starts with `step_`. Then it selects correct method using:

* `Regular expressions`_
* `Format-like strings`_
* `Method names`_

If you look in example from :ref:`usage-guide`:

.. code-block:: python

    # test_acceptance.py

    import unittest

    from morelia import run


    class CalculatorTestCase(unittest.TestCase):

        def test_addition(self):
            ''' Addition feature '''
            filename = os.path.join(os.path.dirname(__file__), 'calculator.feature')
            run(filename, self, verbose=True)

        def step_I_have_powered_calculator_on(self):
            r'I have powered calculator on'
            self.stack = []

        def step_I_enter_a_number_into_the_calculator(self, number):
            r'I enter "(\d+)" into the calculator'  # match by regexp
            self.stack.append(int(number))

        def step_I_press_add(self):  #  matched by method name
            self.result = sum(self.stack)

        def step_the_result_should_be_on_the_screen(self, number):
            r'the result should be "{number}" on the screen'  # match by format-like string
            self.assertEqual(int(number), self.result)

You'll see three types of matching.

Regular expressions
^^^^^^^^^^^^^^^^^^^

Method ``step_I_enter_number_into_the_calculator`` from example is matched
by :py:mod:`regular expression <re>` as it's docstring

.. code-block:: python

        r'I enter "(\d+)" into the calculator'

matches steps:

.. code-block:: cucumber

        When I enter "50" into the calculator
        And I enter "70" into the calculator

Regular expressions, such as ``(\d+)``, are expanded into positional step arguments,
such as ``number`` in above example. If you would use named groups like ``(?P<number>\d+)``
then capttured expressions from steps will be put as given keyword argument to method.

Remember to use tight expressions, such as ``(\d+)``,
not expressions like ``(\d*)`` or ``(.*)``, to validate your input.

Format-like strings
^^^^^^^^^^^^^^^^^^^

Method ``step_the_result_should_be_on_the_screen`` from example is matched
by :py:class:`format-like strings <string.Formatter>` as it's docstring

.. code-block:: python

        r'the result should be "{number}" on the screen'

matches step:

.. code-block:: cucumber

        Then the result should be "120" on the screen

Method names
^^^^^^^^^^^^

Method ``step_I_press_add`` from example is matched by method name which matches
step:

.. code-block:: cucumber

        And I press add

Own matchers
^^^^^^^^^^^^

You can limit matchers for only some types or use your own matchers.
Matcher classes can be passed to :py:func:`morelia.run` method as keyword parameter:

.. code-block:: python

   from morelia.matchers import RegexpStepMatcher
   # ...
   run(filename, self, matchers=[MyOwnMatcher, RegexpStepMatcher])

.. _matching-tables:

Tables
------

If you use Scenarios with tables and `<angles>` around the payload variable names:

.. code-block:: cucumber

    Scenario: orders above $100.00 to the continental US get free ground shipping
      When we send an order totaling $<total>, with a 12345 SKU, to our warehouse
       And the order will ship to <destination>
      Then the ground shipping cost is $<cost>
       And <rapid> delivery might be available

           |  total | destination            |  cost | rapid |

           |  98.00 | Rhode Island           |  8.25 |  yes  |
           | 101.00 | Rhode Island           |  0.00 |  yes  |
           |  99.00 | Kansas                 |  8.25 |  yes  |
           | 101.00 | Kansas                 |  0.00 |  yes  |
           |  99.00 | Hawaii                 |  8.25 |  yes  |
           | 101.00 | Hawaii                 |  8.25 |  yes  |
           | 101.00 | Alaska                 |  8.25 |  yes  |
           |  99.00 | Ontario, Canada        | 40.00 |   no  |
           |  99.00 | Brisbane, Australia    | 55.00 |   no  |
           |  99.00 | London, United Kingdom | 55.00 |   no  |
           |  99.00 | Kuantan, Malaysia      | 55.00 |   no  |
           | 101.00 | Tierra del Fuego       | 55.00 |   no  |

then that Scenario will unroll into a series of scenarios,
each with one value from the table inserted into their placeholders `<total>`,
`<destination>`, and `<rapid>`.
So this step method will receive each line in the "destination" column:

.. code-block:: python

    def step_the_order_will_ship_to_(self, location):
        r'the order will ship to (.*)'

(And observe that naming the placeholder the same as the method argument
is a *reeeally* good idea, but naturally unenforceable.)

Morelia will take each line of the table,
and construct a complete test case out of the Scenario steps,
running :py:meth:`unittest.TestCase.setUp()` and :py:meth:`unittest.TestCase.tearDown()` around them.

If you use many tables then Morelia would use permutation of all rows in all tables:

.. code-block:: cucumber

    Scenario: orders above $100.00 to the continental US get free ground shipping
      When we send an order totaling $<total>, with a 12345 SKU, to our warehouse
       And the order will ship to <destination>
       And we choose that delivery should be <speed>
            | speed   |

            | rapid   |
            | regular |

      Then the ground shipping cost is $<cost>

           |  total | destination            |  cost |

           |  98.00 | Rhode Island           |  8.25 |
           | 101.00 | Rhode Island           |  0.00 |
           |  99.00 | Kansas                 |  8.25 |

In above example 2 * 3 = 6 different test cases would be generated.

.. _matching-docstrings:

Doc Strings
-----------

Docstrings attached to steps are passed as keyword argument `_text` into
method:

.. code-block:: cucumber

    Feature: Addition
        In order to avoid silly mistakes
        As a math idiot
        I want to be told the sum of two numbers

    Scenario: Add two numbers
        Given I have powered calculator on
        When I enter "50" into the calculator
        And I enter "70" into the calculator
        And I press add
        Then I would see on the screen
            '''
            Calculator example
            ==================
             50
            +70
            ---
            120
            '''

.. code-block:: python

   def step_i_would_see_on_the_screen(self, _text):
        pass

   # or

   def step_i_would_see_on_the_screen(self, **kwargs):
        _text = kwargs.pop('_text')

Morelia is smart enough not to passing this argument if you don't name it.
Below example won't raise exception:

.. code-block:: python

   def step_i_would_see_on_the_screen(self):
        pass

It'll be simply assumed that you ignore docstring.

.. _labels-matching:

Labels
------

Labels attached to features and scenarios are available as keyword argument `_label`:

.. code-block:: cucumber

    @web
    @android @ios
    Feature: Addition
        In order to avoid silly mistakes
        As a math idiot
        I want to be told the sum of two numbers

    @wip
    Scenario: Add two numbers
        Given I have powered calculator on
        When I enter "50" into the calculator
        And I enter "70" into the calculator
        And I press add
        Then the result should be "120" on the screen

.. code-block:: python

   def step_I_enter_number_into_the_calculator(self, num, _label):
        pass

As like with doc-strings you can ommit keyword parameter if you don't need it:

.. code-block:: python

   def step_I_enter_number_into_the_calculator(self, num):
        pass

Labels allows you to implement custom logic depending on labels given.

.. note:: **Compatibility**

   Morelia does not connects any custom logic with labels as some other
   Behavior Driven Development tools do. You are put in the charge and should
   add logic if any. If you are looking for ability to selectivly running
   features and scenarios look at :py:func:`morelia.decorators.tags` decorator.

Matchers Classes
----------------
"""

from abc import ABCMeta, abstractmethod
import re
import unicodedata

import parse

from .utils import to_unicode


class IStepMatcher(object):
    """ Matches methods to steps.

    Subclasses should implement at least `match` and `suggest` methods.
    """

    __metaclass__ = ABCMeta

    def __init__(self, suite, step_pattern='^step_'):
        self._suite = suite
        self._matcher = re.compile(step_pattern)
        self._next = None

    def _get_all_step_methods(self):
        match = self._matcher.match
        return [method_name for method_name in dir(self._suite) if match(method_name)]

    def add_matcher(self, matcher):
        """ Add new matcher at end of CoR

        :param IStepMatcher matcher: matcher to add
        :returns: self
        """
        if self._next is None:
            self._next = matcher
        else:
            self._next.add_matcher(matcher)
        return self

    def find(self, predicate, augmented_predicate, step_methods=None):
        if step_methods is None:
            step_methods = self._get_all_step_methods()
        method, args, kwargs = self.match(predicate, augmented_predicate, step_methods)
        if method:
            return method, args, kwargs
        if self._next is not None:
            return self._next.find(predicate, augmented_predicate, step_methods)
        return None, (), {}

    @abstractmethod
    def match(self, predicate, augmented_predicate, step_methods):
        """ Match method from suite to given predicate.

        :param str predicate: step predicate
        :param str augmented_predicate: step augmented_predicate
        :param list step_methods: list of all step methods from suite
        :returns: (method object, args, kwargs)
        :rtype: (method, tuple, dict)
        """
        pass  # pragma: nocover

    def suggest(self, predicate):
        """ Suggest method definition.

        Method is used to suggest methods that should be implemented.

        :param str predicate: step predicate
        :returns: (suggested method definition, suggested method name, suggested docstring)
        :rtype: (str, str, str)
        """
        docstring, extra_arguments = self._suggest_doc_string(predicate)
        method_name = self.slugify(predicate)
        suggest = u'    def step_%(method_name)s(self%(args)s):\n        %(docstring)s\n\n        raise NotImplementedError(\'%(predicate)s\')\n\n' % {
            'method_name': method_name,
            'args': extra_arguments,
            'docstring': docstring,
            'predicate': predicate.replace("'", "\\'"),
        }
        return suggest, method_name, docstring

    def _suggest_doc_string(self, predicate):
        predicate = predicate.replace("'", r"\'").replace('\n', r'\n')

        arguments = self._add_extra_args(r'["\<](.+?)["\>]', predicate)
        arguments = self._name_arguments(arguments)

        predicate = self.replace_placeholders(predicate, arguments)
        predicate = re.sub(r' \s+', r'\s+', predicate)

        arguments = self._format_arguments(arguments)
        return "r'%s'" % predicate, arguments

    def _name_arguments(self, extra_arguments):
        if not extra_arguments:
            return ''
        arguments = []
        number_arguments_count = sum(1 for arg_type, arg in extra_arguments
                                     if arg_type == 'number')
        if number_arguments_count < 2:
            num_suffixes = iter([''])
        else:
            num_suffixes = iter(range(1, number_arguments_count + 1))

        for arg_type, arg in extra_arguments:
            if arg_type == 'number':
                arguments.append('number%s' % next(num_suffixes))
            else:
                arguments.append(arg)
        return arguments

    def _format_arguments(self, arguments):
        if not arguments:
            return ''
        return ', ' + ', '.join(arguments)

    def replace_placeholders(self, predicate, arguments):
        predicate = re.sub(r'".+?"', '"([^"]+)"', predicate)
        predicate = re.sub(r'\<.+?\>', '(.+)', predicate)
        return predicate

    def _add_extra_args(self, matcher, predicate):
        args = re.findall(matcher, predicate)
        result = []
        for arg in args:
            try:
                float(arg)
            except ValueError:
                arg = ('id', self.slugify(arg))
            else:
                arg = ('number', arg)
            result.append(arg)
        return result

    def slugify(self, predicate):
        predicate = to_unicode(predicate)
        result = []
        for part in re.split('[^\w]+', predicate):
            part = unicodedata.normalize('NFD', part).encode('ascii', 'replace').decode('utf-8')
            part = part.replace(u'??', u'_').replace(u'?', u'')
            try:
                float(part)
            except ValueError:
                pass
            else:
                part = 'number'
            result.append(part)
        return '_'.join(result).strip('_')


class MethodNameStepMatcher(IStepMatcher):

    ''' Matcher that matches steps by method name. '''

    def match(self, predicate, augmented_predicate, step_methods):
        ''' See :py:meth:`IStepMatcher.match` '''

        clean = re.sub(r'[^\w]', '_?', predicate)
        pattern = '^step_' + clean + '$'
        regexp = re.compile(pattern)
        step_methods = [method for method in step_methods if regexp.match(method)]
        for method_name in step_methods:
            method = self._suite.__getattribute__(method_name)
            return method, (), {}
        return None, (), {}

    def suggest(self, predicate):
        ''' See :py:meth:`IStepMatcher.suggest` '''

        method_name = self.slugify(predicate)
        suggest = u'    def step_%(method_name)s(self):\n\n        raise NotImplementedError(\'%(predicate)s\')\n\n' % {
            'method_name': method_name,
            'predicate': predicate.replace("'", "\\'"),
        }
        return suggest, method_name, ''

    def slugify(self, predicate):
        predicate = to_unicode(predicate)
        predicate = unicodedata.normalize('NFD', predicate).encode('ascii', 'replace').decode('utf-8')
        predicate = predicate.replace(u'??', u'_').replace(u'?', u'')
        return re.sub(u'[^\w]+', u'_', predicate, re.U).strip('_')


class RegexpStepMatcher(IStepMatcher):

    ''' Matcher that matches steps by regexp in docstring. '''

    def match(self, predicate, augmented_predicate, step_methods):
        ''' See :py:meth:`IStepMatcher.match` '''

        for method_name in step_methods:
            method = self._suite.__getattribute__(method_name)
            doc = method.__doc__
            if not doc:
                continue
            doc = to_unicode(doc)
            doc = re.compile('^' + doc + '$')
            m = doc.match(augmented_predicate)

            if m:
                kwargs = m.groupdict()
                if not kwargs:
                    args = m.groups()
                else:
                    args = ()
                return method, args, kwargs
        return None, (), {}


class ParseStepMatcher(IStepMatcher):

    ''' Matcher that matches steps by format-like string in docstring. '''

    def match(self, predicate, augmented_predicate, step_methods):
        ''' See :py:meth:`IStepMatcher.match` '''

        for method_name in step_methods:
            method = self._suite.__getattribute__(method_name)
            doc = method.__doc__
            if not doc:
                continue
            match = parse.parse(doc, augmented_predicate)
            if match:
                args = match.fixed
                kwargs = match.named
                return method, tuple(args), kwargs
        return None, (), {}

    def replace_placeholders(self, predicate, arguments):
        arguments = iter(arguments)

        def repl(match):
            if match.group(0).startswith('"'):
                return '"{%s}"' % next(arguments)
            return '{%s}' % next(arguments)

        predicate = re.sub(r'".+?"|\<.+?\>', repl, predicate)
        return predicate



import sys

import six


def to_unicode(text):
    ''' Try convert to unicode independently on python version. '''
    try:
        text = text.decode('utf-8')
    except (UnicodeDecodeError, UnicodeEncodeError, AttributeError):
        pass
    return text


if six.PY2:

    def fix_exception_encoding(exc):
        if len(exc.args):
            message = exc.args[0]
            if isinstance(message, unicode):
                exc.args = (message.encode(sys.stderr.encoding or "ascii", "xmlcharrefreplace"),) + exc.args[1:]

    def to_docstring(text):
        return text.encode('utf-8')

else:

    def fix_exception_encoding(exc):
        pass

    def to_docstring(text):
        return text



import os.path
import re
from unittest import TestCase

from morelia.decorators import tags
from morelia.parser import Parser

pwd = os.path.dirname(os.path.realpath(__file__))


class SampleTestCaseMixIn(object):

    def runTest(self):
        pass  # pragma: no cover

    def step_I_have_powered_calculator_on(self):
        r'I have powered calculator on'

        self.stack = []

    def step_I_enter_number_into_the_calculator(self, number):
        r'I enter "([^"]+)" into the calculator'

        self.stack.append(int(number))

    def step_I_press_add(self):
        r'I press add'

        num1 = self.stack.pop()
        num2 = self.stack.pop()
        self.stack.append(num1 + num2)

    def step_the_result_should_be_number_on_the_screen(self, number):
        r'the result should be "([^"]+)" on the screen'

        self.assertEqual(self.stack[-1], int(number))

    def step_I_press_subtract(self):
        r'I press subtract'

        num1 = self.stack.pop()
        num2 = self.stack.pop()
        self.stack.append(num2 - num1)

    def step_I_press_multiply(self):
        r'I press multiply'

        num1 = self.stack.pop()
        num2 = self.stack.pop()
        self.stack.append(num2 * num1)

    def step_I_press_divide(self):
        r'I press divide'

        num1 = self.stack.pop()
        num2 = self.stack.pop()
        self.stack.append(num2 / num1)


@tags(['acceptance'])
class InfoOnAllFailingScenariosTest(TestCase):

    def test_should_report_on_all_failing_scenarios(self):
        self._add_failure_pattern = re.compile('Scenario: Add two numbers\n\s*Then the result should be "120" on the screen\n\s*.*AssertionError:\s*70 != 120', re.DOTALL)
        self._substract_failure_pattern = re.compile('Scenario: Subtract two numbers\n\s*Then the result should be "80" on the screen\n\s*.*AssertionError:\s*70 != 80', re.DOTALL)
        self._multiply_failure_pattern = re.compile('Scenario: Multiply two numbers\n\s*Then the result should be "12" on the screen\n\s*.*AssertionError:\s*3 != 12', re.DOTALL)
        self._division_failure_pattern = re.compile('Scenario: Divide two numbers\n\s*Then the result should be "4" on the screen\n\s*.*AssertionError:\s*2 != 4', re.DOTALL)
        filename = os.path.join(pwd, 'features/info_on_all_failing_scenarios.feature')
        ast = Parser().parse_file(filename)
        ast.evaluate(self)

    def step_feature_with_number_scenarios_has_been_described_in_file(self, feature_file):
        r'that feature with 4 scenarios has been described in file "([^"]+)"'
        filename = os.path.join(pwd, 'features/{}'.format(feature_file))
        self._ast = Parser().parse_file(filename)

    def step_that_test_case_passing_number_and_number_scenario_and_failing_number_and_number_has_been_written(self):
        r'that test case passing 1 and 3 scenario and failing 2 and 4 has been written'

        class _EvaluatedTestCase(SampleTestCaseMixIn, TestCase):

            def step_I_press_subtract(self):
                pass

            def step_I_press_divide(self):
                pass

        self._evaluated_test_case = _EvaluatedTestCase
        self._failing_patterns = [
            self._substract_failure_pattern,
            self._division_failure_pattern
        ]

    def step_that_test_case_failing_all_scenarios_been_written(self):
        r'that test case failing all scenarios been written'

        class _EvaluatedTestCase(SampleTestCaseMixIn, TestCase):

            def step_I_press_add(self):
                pass

            def step_I_press_subtract(self):
                pass

            def step_I_press_multiply(self):
                pass

            def step_I_press_divide(self):
                pass

        self._evaluated_test_case = _EvaluatedTestCase
        self._failing_patterns = [
            self._add_failure_pattern,
            self._substract_failure_pattern,
            self._multiply_failure_pattern,
            self._division_failure_pattern
        ]

    def step_that_test_case_passing_number_number_and_number_scenario_and_failing_number_has_been_written(self):
        r'that test case passing 2, 3 and 4 scenario and failing 1 has been written'

        class _EvaluatedTestCase(SampleTestCaseMixIn, TestCase):

            def step_I_press_add(self):
                pass

        self._evaluated_test_case = _EvaluatedTestCase
        self._failing_patterns = [
            self._add_failure_pattern,
        ]

    def step_that_test_case_passing_all_scenarios_been_written(self):
        r'that test case passing all scenarios been written'

        class _EvaluatedTestCase(SampleTestCaseMixIn, TestCase):
            pass

        self._evaluated_test_case = _EvaluatedTestCase

    def step_that_test_case_failing_number_number_and_number_scenario_and_passing_number_has_been_written(self):
        r'that test case failing 1, 2 and 3 scenario and passing 4 has been written'

        class _EvaluatedTestCase(SampleTestCaseMixIn, TestCase):

            def step_I_press_add(self):
                pass

            def step_I_press_subtract(self):
                pass

            def step_I_press_multiply(self):
                pass

        self._evaluated_test_case = _EvaluatedTestCase
        self._failing_patterns = [
            self._add_failure_pattern,
            self._substract_failure_pattern,
            self._multiply_failure_pattern,
        ]

    def step_I_run_test_case(self):
        r'I run test case'

        self._catch_exception = None
        try:
            tc = self._evaluated_test_case()
            self._ast.evaluate(tc)
        except Exception as e:
            self._catch_exception = e  # warning: possible leak, use with caution

    def step_I_will_get_assertion_error_with_information_number_scenarios_passed_number_scenarios_failed(self, message):
        r'I will get assertion error with information "([^"]+)"'

        message = self._catch_exception.args[0]
        self.assertTrue(message.startswith(message))

    def step_I_will_get_traceback_of_each_failing_scenario(self):
        r'I will get traceback of each failing scenario'

        patterns = self._failing_patterns
        message = self._catch_exception.args[0]
        for pattern in patterns:
            self.assertRegexpMatches(message, pattern)

    def step_I_won_t_get_assertion_error(self):
        r'I won\'t get assertion error'

        self.assertIsNone(self._catch_exception)



import unittest
from collections import OrderedDict
from mock import Mock, sentinel, ANY

from morelia.decorators import tags
from morelia.exceptions import MissingStepError
from morelia.grammar import Step
from morelia.visitors import StepMatcherVisitor, TestVisitor


@tags(['unit'])
class StepMatcherVisitorVisitTestCase(unittest.TestCase):
    """ Test :py:meth:`StepMatcherVisitor.visit`. """

    def test_should_find_step(self):
        """ Scenario: step found """
        # Arrange
        obj = StepMatcherVisitor(sentinel.suite, sentinel.matcher)
        node = Mock()
        # Act
        obj.visit(node)
        # Assert
        node.find_step.assert_called_once_with(sentinel.matcher)

    def test_should_not_find_step(self):
        """ Scenario: step not found """
        # Arrange
        obj = StepMatcherVisitor(sentinel.suite, sentinel.matcher)
        node = Mock()
        exc = MissingStepError('predicate', sentinel.suggest,
                               sentinel.method_name, sentinel.docstring)
        node.find_step.side_effect = [exc]
        # Act
        obj.visit(node)
        # Assert
        node.find_step.assert_called_once_with(sentinel.matcher)
        self.assertTrue(sentinel.docstring in obj._not_matched)

    def test_should_not_find_step_by_method_name_matcher(self):
        """ Scenario: step not found by method name matcher """
        # Arrange
        obj = StepMatcherVisitor(sentinel.suite, sentinel.matcher)
        node = Mock()
        exc = MissingStepError('predicate', sentinel.suggest,
                               sentinel.method_name, '')
        node.find_step.side_effect = [exc]
        # Act
        obj.visit(node)
        # Assert
        node.find_step.assert_called_once_with(sentinel.matcher)
        self.assertTrue(sentinel.method_name in obj._not_matched)


@tags(['unit'])
class StepMatcherVisitorReportMissingTestCase(unittest.TestCase):
    """ Test :py:meth:`StepMatcherVisitor.report_missing`. """

    def test_should_fail_with_suggest(self):
        """ Scenario: fail with suggset """
        # Arrange
        suite = Mock()
        obj = StepMatcherVisitor(suite, sentinel.matcher)
        obj._not_matched = OrderedDict([('docstring', 'suggest string')])
        # Act
        obj.report_missing()
        # Assert
        suite.fail.assert_called_once_with(u'Cannot match steps:\n\nsuggest string')

    def test_should_not_fail(self):
        """ Scenario: no steps missing """
        # Arrange
        suite = Mock()
        obj = StepMatcherVisitor(suite, sentinel.matcher)
        obj._not_matched = OrderedDict()
        # Act
        obj.report_missing()
        # Assert
        self.assertFalse(suite.fail.called)


@tags(['unit'])
class TestVisitorVisitTestCase(unittest.TestCase):
    """ Test :py:meth:`TestVisitor.visit`. """

    def test_should_catch_SystemExit(self):
        """ Scenario: SystemExit """
        # Arrange
        formatter = Mock()
        node = Mock(Step)
        suite = Mock()
        obj = TestVisitor(suite, sentinel.matcher, formatter)
        node.find_step.side_effect = [SystemExit]
        # Act
        # Assert
        self.assertRaises(SystemExit, obj.visit, node)
        formatter.output.assert_called_once_with(node, ANY, 'error', ANY)
